function pval = xarsieve(y, B)

%=============================================================
% INFOBOX
%=============================================================
% PURPOSE:
% calculates the bootstrap-based p-value of the Anderson-Darling test statistic
% for normality
%
% INPUT:
%   y = (n x 1) vector of obseravtions
%   B = a number of bootstrap replications
%
% OUTPUT:
%   pval
%
% AUTHOR: Marian Vavra
%
% References:
% [1] Psaradakis and Vavra (2017), Econometrics and Statistics
% [2] Psaradakis (2003), Communications in Statistics B, 32
%=============================================================

% the sample estimate of the AD statistic from observed data
ad0 = xadtest(y);

% AR filter for a sieve bootstrap including inflation and symmetrization of the estimated
% residuals
n      = length(y);
pmax   = floor((log(n))^2);
p      = xautolag(y,'AIC',pmax);
Ylag   = mlag(y,p,1);
Y      = y(p+1:end);
[Q1,R1]= qr(Ylag,0);
phi    = R1\(Q1'*Y);
e      = Y - Ylag*phi;
su     = sqrt((e'*e)/(length(e)-2*p-1));

% sieve bootstrap
N   = n+100;
for ii = 1:B
    eb = random('normal',0,su,N,1);
    yb = [mean(y)*ones(p,1); zeros(N-p,1)];
    
    for t=p+1:N
        yb(t) = phi'*[1;yb(t-1:-1:t-p)] + eb(t);
    end
    yb = yb(101:end);
    adb(ii,1) = xadtest(yb);
    
end

pval = mean(adb > ad0);

